@@ -14,7 +14,7 @@ exams2blackboard <- function(file, n = 1L, nsamp = NULL, dir = ".",
     converter <- if(any(tolower(tools::file_ext(unlist(file))) == "rmd")) "pandoc" else "ttm"
   }
   ## set up .html transformer
-  htmltransform <- make_exercise_transform_html(converter = converter, ...)
+  htmltransform <- make_exercise_transform_html(converter = converter, base64 = base64, ...)
 
   ## generate the exam
   exm <- xexams(file, n = n, nsamp = nsamp,
