@@ -208,6 +208,22 @@ exams_metainfo <- function(x, ...) {
     class = "exams_metainfo")
 }
 
+print.exams_metainfo <- function(x, which = NULL, block = NULL, ...) {
+  which <- if(is.null(which)) names(x) else {
+    if(is.numeric(which)) names(x)[which] else which
+  }
+  n <- length(x[[1L]])
+  for(i in which) {
+    cat("\n", i, "\n", sep = "")
+    for(j in 1L:n) {
+      cat("    ", format(c(n, j))[-1L], ". ", x[[i]][[j]]$string, "\n", sep = "")
+      if(!is.null(block) && j %% as.integer(block) == 0L) cat("\n")
+    }
+  }
+  cat("\n")
+  invisible(x)
+}
+
 xweave <- function(file, quiet = TRUE, encoding = NULL, engine = NULL,
   envir = new.env(), pdf = TRUE, png = FALSE, svg = FALSE, height = 6, width = 6,
   resolution = 100, highlight = FALSE, ...)
